/*********************************************************************
	deskimg
	
	This program exposes a replicatable view that displays a picture.
	The purpose is to replace the DesktopImage functionality that
	we had before AA and its funky desktop-window. There's room for
	improvement, I'm sure.
	
	Who		When		What
	GJW		970723	Wrote the whole thing but couldn't figure out
					how to make the view replicatable
	GJW		970725	Figured out the problem. My instantiation
					function was being linked out. Fixed it up,
					documented the code like I promised people I
					would, and packaged it up for Be.
*********************************************************************/

//	Headers
#include "Datatypes.h"	//	From Jon Watte's DataTypes distribution
#include "GetBitmap.h"	//	From Jon Watte's DataTypes distribution
#include "deskimg.h"

//	Constants
const char *app_signature = "application/x-vnd.Bastion-deskimg";


/*********************************************************************
	Instantiate
	Args:	inData - contains an archived object to extract
	Returns:	a DeskImage as described in the archive
	This static function examines the archive to see if it contains
	something that could conceivably be a DeskImage. If not, we
	return NULL. Otherwise, we construct the object from the archive
	and give that back to the caller.
*********************************************************************/
DeskImage* DeskImage::Instantiate(BMessage* inData)
{
	if(!validate_instantiation(inData,"DeskImage")) return NULL;
	return new DeskImage(inData);
}


/*********************************************************************
	DeskImage
	Args:	<none>
	Returns:	<none>
	The default constructor initializes its superclass, and creates
	a dragger to handle the replicant UI. We also call a private
	function to try to load the picture.
*********************************************************************/
DeskImage::DeskImage(void)
	: BView(BRect(0,0,16,16),"DeskImage",B_FOLLOW_NONE,B_WILL_DRAW)
{
	InitObject();

	AddChild(new BDragger(BRect(0,0,7,7),this,B_FOLLOW_RIGHT | B_FOLLOW_BOTTOM));
}


/*********************************************************************
	DeskImage
	Args:	inData - an archived DeskImage
	Returns:	<none>
	This constructor initializes its superclass with the data in
	the passed archive, and then calls a private function to load
	the picture we will display.
*********************************************************************/
DeskImage::DeskImage(BMessage* inData)
	: BView(inData)
{
	InitObject();
}


/*********************************************************************
	~DeskImage
	Args:	<none>
	Returns:	<none>
	The destructor tosses the bitmap if it exists.
*********************************************************************/
DeskImage::~DeskImage(void)
{
	if(mImage != NULL) delete mImage;
}


/*********************************************************************
	Archive
	Args:	inData - the message to archive the view into
			deep - indicates whether to include sub-objects
	Returns:	a result code indicating success or reason for failure
	Call the superclass archive message and then add our own info.
	We don't store the bitmap; it's supposed to be in the user's
	tree somewhere.
*********************************************************************/
status_t DeskImage::Archive(BMessage *inData, bool deep) const
{
	status_t result = inherited::Archive(inData, deep);
	
	if(result == B_OK)
	{
		inData->AddString("class", "DeskImage");
		inData->AddString("add_on",app_signature);
	}
	
	return result;
}


/*********************************************************************
	AttachedToWindow
	Args:	<none>
	Returns:	<none>
	Handles placement in a window by moving the view to the upper-
	left corner and resizing to fill the window. This function should
	be modified to support other placement options.
	
	Commented out because the BeOS desktop window is a completely
	funky size when replicants get added to it.
*********************************************************************/
void DeskImage::AttachedToWindow(void)
{
//	BRect theRect = Bounds();
//	BRect wndRect = Window()->Bounds();
	
//	float x = (wndRect.Width() - theRect.Width()) / 2;
//	float y = (wndRect.Height() - theRect.Height()) / 2;
//	MoveTo(x,y);
}


/*********************************************************************
	Draw
	Args:	inUpdate - ignored
	Returns:	<none>
	If we have a bitmap, we draw it in the upper-left corner of the
	view. If not, we draw a big X across it. This function should be
	modified to support other placement options.
*********************************************************************/
void DeskImage::Draw(BRect /*inUpdate*/)
{
	BRect theRect = Bounds();

	if(mImage == NULL)
	{
		FillRect(theRect,B_SOLID_LOW);
		StrokeLine(theRect.LeftTop(),theRect.RightBottom());
		StrokeLine(theRect.LeftBottom(),theRect.RightTop());
	}
	else
	{
		DrawBitmap(mImage,BPoint(0,0));
	}
}


/*********************************************************************
	InitObject
	Args:	<none>
	Returns:	<none>
	Private utility that attempts to load the image to display.
*********************************************************************/
void DeskImage::InitObject(void)
{
	//	Make sure it doesn't look like we have an image
	mImage = NULL;

	//	Try to load the image
	if(DATAInit(app_signature) == B_OK)
	{
		BPath thePath;
		status_t err = find_directory(B_USER_CONFIG_DIRECTORY,&thePath);
		thePath.Append("DesktopImage");
		mImage = GetBitmap(thePath.Path());
		DATAShutdown();
	}
	
	if(mImage != NULL)
	{
		BRect theRect = mImage->Bounds();
		ResizeTo(theRect.Width(),theRect.Height());
	}
}


/*********************************************************************
	WinImage
	This is trivial enough to get a block comment all to itself.
	This window gives you a place to get the replicant from, and
	suggests to the app that it quit when the window is closed.
*********************************************************************/
class WinImage : public BWindow {
public:
	WinImage(void)
		: BWindow(BRect(100,50,250,200),"Desk Image",B_TITLED_WINDOW,0)
	{
		AddChild(new DeskImage);
	}
	
	virtual ~WinImage(void)
	{
		/* This space intentionally left blank */
	}
	
	virtual bool QuitRequested(void)
	{
		be_app->PostMessage(B_QUIT_REQUESTED);
		return true;
	}
};


/*********************************************************************
	main
	args:	<none>
	returns:	zero
	Just a simple little main that creates what we need and gives
	it a push to get it going.
*********************************************************************/
int main(void)
{
	BApplication app(app_signature);
	WinImage* win = new WinImage;
	win->Show();
	app.Run();
	return 0;
}